#include <iostream>
#include <readline/readline.h>
#include <readline/history.h>
using namespace std; 

#include "mystd.h"
#include "asi.h"
#include "gar.h"

// *** decode routines ***

char bask_decode( const string& resp, bool *rval )
{
    if        ( resp == "" ) {
        return 'd';
    } else if ( resp == "y" ) {
        *rval = 1;
        return 'v';
    } else if ( resp == "n" ) {
        *rval = 0;
        return 'v';
    } else
        return 'h';
}

char iask_decode( const string& resp, int *rval )
{
    int rexp, i;
    if ( resp=="" ) {
        return 'd';
    } else if (( sscanf(resp.c_str(), "%de%d", rval, &rexp)==2 ||
                 sscanf(resp.c_str(), "%dE%d", rval, &rexp)==2 ) &&
               rexp>=0 ) {
        for ( i=0; i<rexp; ++i ) { *rval *= 10; }
        return 'v';
    } else if (sscanf(resp.c_str(), "%d", rval)==1) {
        return 'v';
    } else
        return 'h';
}

char dask_decode( const string& resp, double *rval )
{
    if        ( resp=="" ) {
        return 'd';
    } else if ( !mystd::any2dbl( resp, rval) ) {
        return 'v';
    } else
        return 'h';
}

char sask_decode( const string& resp, string *rval )
{
    if        ( resp=="" ) {
        return 'd';
    } else if ( resp=="\\EOL" ) {
        *rval = string("");
        return 'v';
    } else if ( resp=="\\?" ) {
        return 'h';
    } else {
        *rval = resp;
        return 'v';
    }
}

// *** intermediate calls for simple types: ***

const char *bask_helptext = "a boolean value (y or n)";
const char *iask_helptext = "an integer number";
const char *dask_helptext = "a floating-point number";
const char *sask_helptext = "a string";

bool   bask( const string& quest, const int defflag, const bool& defval)
{
    return (bool) myask( quest, defflag | ASK_WORD, defval, 
                         bask_helptext, bask_decode );
}

int    iask( const string& quest, const int defflag, const int& defval)
{
    return (int) myask( quest, defflag | ASK_WORD, defval,
                        iask_helptext, iask_decode );
}

double dask( const string& quest, const int defflag, const double& defval)
{
    return (double) myask( quest, defflag | ASK_WORD, defval,
                           dask_helptext, dask_decode );
}

string wask( const string& quest, const int defflag, const string& defval)
{
    return (string) myask( quest, defflag | ASK_WORD, defval,
                           sask_helptext, sask_decode );
} // ACHTUNG: umstellung auf wask_ unfertig :ACHTUNG

string sask( const string& quest, const int defflag, const string& defval)
{
    return (string) myask( quest, defflag, defval, 
                           sask_helptext, sask_decode );
}
