//**************************************************************************//
//* FRIDA: fast reliable interactive data analysis                         *//
//* index.cpp: integer lists containing indices                            *//
//* (C) Joachim Wuttke 1990-, v2(C++) 2001-                                *//
//* http://frida.sourceforge.net                                           *//
//**************************************************************************//

#include <stdio.h>
#include <string.h>
#include <iostream>
#include <algorithm>
using namespace std;

#include "mystd.h"
#include "index.h"
#include "gar.h"   // for ask

namespace IndexParse
{
	enum bla { TEST, LIST, SET } client;
	void* saveto;
	void newval(uint val);
	uint p0, pn;

	bool ParseTest(string inp, uint v0=0, uint vn=0);
	bool ParseSet(set<uint> *T, string inp, uint v0=0, uint vn=0);
	bool ParseList(vector<uint> *V, string inp, uint v0=0, uint vn=0);
	bool ParseExe(string inp);
	bool ParseSkipinterval(string inp);
	bool ParseSkipfactor(string inp, int *val);
	bool ParseInterval(string inp, int skip);
	bool ParseValue(string inp, int which, uint *val);

	inline bool onlytest() { return client==TEST; };
	inline bool limgiven() { return (pn > 0); };
	inline bool allowoutoflim() { return client==TEST; }
}

namespace IndexDialog {
	extern const char* helptext;
	char decode(const string& resp, string *rval );
}


// ******* unsorted list *******

// *** modify ***

IndexList IndexList::append(const IndexList& L)
{ 
        for (uint i=0; i<L.V.size(); ++i)
                V.push_back(L.V[i]);
        return *this;
}

IndexList IndexList::append(const uint val)
{
	V.push_back(val);
	return *this;
}

IndexList IndexList::clear(void)
{ 
	V.clear();
	return *this;
}

IndexList IndexList::reverse(void)
{
	std::reverse(V.begin(), V.end());
	return *this;
}

IndexList IndexList::sort(void)
{ 
	std::sort(V.begin(), V.end());
	return *this;
}

IndexList IndexList::complement(const IndexList Vec2, 
				const uint v0, const uint vn)
{
	clear();
	IndexList VecS(Vec2);
	VecS.sort();
	uint iS, v;
	for (iS=0; iS<VecS.size() && VecS.V[iS]<v0; iS++);
	for (v=v0; v<vn; ++v) {
		if (iS>=VecS.size()) break; 
		if (VecS.V[iS]==v)
			iS++;
		else
			V.push_back(v);
	}
	for (; v<vn; ++v) 
		V.push_back(v);
	return *this;
}

// *** from text ***

bool IndexList::parse(const string s, const uint v0, const uint vn)
{
	return (IndexParse::ParseList(&V, s, v0, vn));
}

// *** info ***

bool IndexList::contained(const uint val) const
{
	for (uint i=0; i<size(); i++)
		if (val==V[i]) return 1;
	return 0;
}

string IndexList::str(void) const
{
	if(!V.size()) {
		return "-";
	}
	string s;
        vector<uint> VV=V;
	vector<uint>::iterator iV;
        iV=VV.begin();
	uint vi, vd;
	while(iV<V.end()) {
		s += strg(vi=*(iV++));
		if(iV>=V.end()) break;
		vd = *(iV++) - vi;
		if(iV>=V.end()) {
			s += "," + strg(vi+vd);
			break;
		}
		if (*iV!=vi+2*vd) {
			s += ",";
			iV--;
			continue;
		}
		while(iV<V.end() && *(iV-1)+vd==*iV) iV++;
		s += "-" + strg(*(iV-1));
		if (vd!=1) s += "." + strg(vd);
		if(iV>=V.end()) break;
		s += ",";
	}
	return s;
}


// ******* sorted list *******

// *** modify ***

IndexSet IndexSet::insert(const IndexSet Set)
{
	IndexSetIterator II(Set);
	uint val;
	while(II(&val)) T.insert(val);
	return *this;
}

IndexSet IndexSet::insert(const uint val)
{
	T.insert(val);
	return *this;
}

IndexSet IndexSet::clear(void)
{ 
	T.clear();
	return *this;
}

IndexSet IndexSet::complement(const IndexSet Set2, 
				    const uint v0, const uint vn)
{
	clear();
	IndexSetIterator II(Set2);
	uint v2;
	uint v = v0;
	while (II(&v2)) {
		while (v<v2 && v<vn) T.insert(v++);
		if (v>=vn) return *this;
		v++; // v is now =v2+1
	}
	while (v<vn) T.insert(v++);
	return *this;
}

// *** from text ***

bool IndexSet::parse(const string s, const uint v0, const uint vn)
{
	bool ret = IndexParse::ParseSet(&T, s, v0, vn);
	// printf("DEBUG %u %u [%s] %d\n", v0, vn, s.c_str(), ret);
	if (ret) clear();
	return ret;
}

bool IndexSet::ask(const string quest, const string def, 
		       const uint v0, const uint vn)
{
	string ans = index_ask(quest, def, v0, vn);
	return parse(ans, v0, vn);
}

// *** info ***

bool IndexSet::contained(const uint val) const
{
	return (T.find(val)!=T.end());
}

string IndexSet::str(void) const
{
	if(!T.size()) {
		return "-";
	}
	string s;
	set<uint>::iterator pos=T.begin();
	uint vi, vd;
	while(pos!=T.end()) {
		s += strg(vi=*(pos++));
		if(pos==T.end()) break;
		vd = *(pos++) - vi;
		if(pos==T.end()) {
			s += "," + strg(vi+vd);
			break;
		}
		if (*pos!=vi+2*vd) {
			s += ",";
			pos--;
			continue;
		}
		while(pos!=T.end() && *(--pos)+vd==*(++pos)) pos++;
		s += "-" + strg(*((--pos)++));
		if (vd!=1) s += "." + strg(vd);
		if(pos==T.end()) break;
		s += ",";
	}
	return s;
}


// ******* parse a set/list string *******

// *** interfaces ***

bool IndexParse::ParseTest(string inp, uint v0, uint vn)
{
	p0 = v0; pn = vn;
	client = TEST;
	saveto = 0;
	return ParseExe(inp);
}

bool IndexParse::ParseSet(set<uint> *T, string inp, uint v0, uint vn)
{
	p0 = v0; pn = vn;
	client = SET;
	saveto = (void *) T;
	T->clear();
	return ParseExe(inp);
}

bool IndexParse::ParseList(vector<uint> *V, string inp, uint v0, uint vn)
{
	p0 = v0; pn = vn;
	client = LIST;
	saveto = (void *) V;
	V->clear();
	return ParseExe(inp);
}

void IndexParse::newval(uint val)
{
	switch (client) {
	case TEST:
		return;
	case LIST:
		((vector<uint> *) saveto)->push_back(val);
		return;
	case SET:
		((set<uint> *) saveto)->insert(val);
		return;
	}
}

// *** parse the input string ***

#define ERRLII cout << "! invalid input: "

bool IndexParse::ParseExe(string inp)
{
	if(inp==string("-")) return 0; // valid empty list
	string::size_type ji=0, jf;
	while (ji<inp.size()) {
		if ((jf=inp.find(',', ji)) == string::npos) jf = inp.size();
		if (ParseSkipinterval(inp.substr(ji,jf-ji))) return 1;
		ji = jf+1;
	}
	return 0;
}

bool IndexParse::ParseSkipinterval(string inp)
{
	// cout << inp << endl;
	int skip;
	string::size_type j;
	if ((j=inp.find('.')) != string::npos) {
		if (ParseSkipfactor(inp.substr(j+1), &skip)) return 1;
	} else {
		skip = 1;
		j = inp.size();
	}
	return ParseInterval(inp.substr(0,j), skip);
}

bool IndexParse::ParseSkipfactor(string inp, int *skip)
{
	char dummy;
	if (sscanf(inp.c_str(), "%d%c", skip, &dummy)!=1 || *skip==0) {
		ERRLII "invalid skip factor [" << inp << "]\n";
		*skip = 0;
		return 1;
	}
	return 0;
}

bool IndexParse::ParseInterval(string inp, int skip)
{
	// cout << "intv: " << inp << endl;
	string::size_type j;
	uint vi, vf;
	bool xv=0; // vf exists ?
	if (inp==string("*")) {
		if (!limgiven()) return !onlytest();
		vi = p0;
		vf = p0+pn-1;
		xv = 1;
	} else if ((j=inp.find('-')) != string::npos) {
		if (ParseValue(inp.substr(j+1), +1, &vf)) return 1;
		if (ParseValue(inp.substr(0,j), -1, &vi)) return 1;
		xv = 1;
	} else {
		if (ParseValue(inp, 0, &vi)) return 1;
		vf = vi;
	}
	if(skip>0 && vi>vf) {
		ERRLII "invalid interval [" << inp << "]\n";
		return 1;
	} else if (skip==0) {
		ERRLII "PROGRAM ERROR (skip=0 should not arrive in ParseIntv\n";
		return 1;
	} else if (skip<0 && vi<vf) {
		printf("! ascending interval [%s] with skip=%u<0\n",
		       inp.c_str(), skip);
		return 1;
	}

	if (onlytest()) return 0;
	if (skip>0)
		for (uint vv=vi; vv<=vf; vv += skip)
			newval(vv);
	else
		for (uint vv=vf; vv>=vi; vv += skip)
			newval(vv);
	return 0;
}	

bool IndexParse::ParseValue(string inp, int which, uint *val)
{
	char dummy;
	if (inp==string("*")) {
		if (!limgiven()) return !onlytest();
		if      (which==-1) 
			*val = p0;
		else if (which==+1) 
			*val = p0+pn-1;
		else {
			ERRLII "list value * out of context\n";
			return 1;
		}
	} else if (sscanf(inp.c_str(), "%d%c", val, &dummy)!=1 || *val<0) {
		ERRLII "invalid list value [" << inp << "]\n";
		return 1;
	}
	// printf("DEBUG %d %d [%d %d] %d\n", 
	//        limgiven(), allowoutoflim(), 0, pn, *val);
	if (limgiven() && !allowoutoflim() && (p0>*val || *val>=p0+pn)) {
		ERRLII "list value [" << inp << "] out of bounds " 
				      << "(" << p0 << "," << pn << "( \n";
		return 1;
	}
	return 0;
}

// *** user interaction ***

const char * IndexDialog::helptext = 
	 "a list (a set of indices):\n"
         "    list:            <empty_list> | <finite_list> | <infinite_list>\n"
         "    empty_list:      -\n"
         "    finite_list:     <range_with_skip> { , <range_with_skip> }\n"
         "    infinite_list:   *\n"  
         "    range_with_skip: <range> [ . <skip> ]\n"
         "    range:           <bound> [ - <bound>]\n"
	 "    skip:            (integer >= 1)\n"
	 "    bound:           * | (integer >= 0)\n";

char IndexDialog::decode(const string& resp, string *rval )
{
	if ( resp=="" ) {
		return 'd';
	} else if ( resp=="?" ) {
		return 'h';
	}
	*rval = resp;
	if (!(IndexParse::ParseTest(*rval, 0, 0))) // 0,0 Altlast(Constraints)
		return 'v';
	else
		return 'h';
}

string index_ask( const string& quest, string def, uint p0, uint pn )
{
	uint lims[2];
	lims[0] = p0;
	lims[1] = pn;
	int defflag = ASK_DEF_GIVEN*(def!="") | ASK_WORD;
	return  myask( quest, defflag, def, 
                       IndexDialog::helptext, IndexDialog::decode ); 
}

