%{
//**************************************************************************//
//* FRIDA: fast reliable interactive data analysis                         *//
//* xax.l: lexer for eXtended Array eXpressions                            *//
//* (C) Joachim Wuttke 1990-, v2(C++) 2001-                                *//
//* http://frida.sourceforge.net                                           *//
//**************************************************************************//
%}

%option noyywrap

%{
using namespace std;
#include <boost/shared_ptr.hpp>
#include <math.h>
#include <string>
#include <vector>
#include "func.h"
#include "expr.h"
#include "xax_yacc.h"

// xaxtype must also be declared in xax.y
struct xaxtype {
    double  v;
    void*   p; // for strings and functions
    PTree   t;
    PRef    r;
};
#define YYSTYPE xaxtype

#define YY_DECL int xaxlex(YYSTYPE *xaxlval)
%}

DIG      [0-9]
ID       [a-zA-Z][a-zA-Z0-9]*
EXP      [eE]"-"?[0-9]+
%%

{DIG}+	{
	// printf( "lex: An integer: %s (%d)\n", xaxtext, atoi( xaxtext ) );
	xaxlval->v = atoi(xaxtext);
	return NUM; }

({DIG}+"."{DIG}*{EXP}?)|("."?{DIG}*{EXP}?) {
	// printf( "lex: A float: %s (%g)\n", xaxtext, atof( xaxtext ) );
	xaxlval->v = atof(xaxtext);
	return NUM; }

[xykji]|n[ij]|z{DIG}+|p{DIG}+|fm{DIG}+ {     
        // printf( "lex: The variable %s\n", xaxtext );
        xaxlval->p = (void*) xaxtext; 
        return REF; }

t {
        // printf( "lex: The formal function argument %s\n", xaxtext );
        return ARG; }

f {     
        // printf( "lex: The variable %s\n", xaxtext );
        xaxlval->p = (void*) xaxtext; 
        return FRF; }

pi {
	// printf( "lex: pi\n" );
	xaxlval->v = acos(-1.);
	return NUM; }

e {
	// printf( "lex: e\n" );
	xaxlval->v = exp(1.);
	return NUM; }

"=="  { return BOP_EQ; }
"!="  { return BOP_NE; }
"<"   { return BOP_LT; }
"<="  { return BOP_LE; }
">"   { return BOP_GT; }
">="  { return BOP_GE; }
"&&"  { return BOP_AND; }
"||"  { return BOP_OR; }
"xor" { return BOP_XOR; }
"mod" { return BOP_MOD; } /* man 3 fmod */
"div" { return BOP_DIV; } /* integer division */
"over" { return BOP_OVER; } /* binomial coefficient: n over k */

"conv" { return CONV; }
"dirac" { return DIRAC; }

{ID}	{
        // printf( "lex: An identifier: %s\n", xaxtext );
	CFunc *f1;
        if (!(f1= NFunctions::find(xaxtext))) {
	    printf("ERROR(lex) identifier %s is not a function\n", xaxtext);
	    return 0;
        }
	xaxlval->p = (void*) f1;
	return FNCT; }

"{"[^}\n]*"}"     /* eat up one-line comments */

[ \t\n]+          /* eat up whitespace */ 

"!"|"-"|"+"|"*"|"/"|"%"|"^"|"="|"("|")"|"["|"]"|","|"?"|":" {           
	// printf( "lex: Special character: %s\n", xaxtext );
	return xaxtext[0]; }

.       {           
	printf( "lex: invalid special character: %s\n", xaxtext);
	return 0; }

<<EOF>> { return END; }
%%

int xaxparse( void* );

int user_xaxparse( const char *lin, boost::shared_ptr<class CTree> *T )
{
	int err;
	xax_scan_string(lin);
	err=xaxparse((void*) T);
	xax_delete_buffer(YY_CURRENT_BUFFER);
	return err;
}
